#if !defined(__DESKTOPITEM_H)
#define __DESKTOPITEM_H

class Font;
class Texture;

class DesktopItem
{
public:

	DesktopItem(const string &name);
	virtual ~DesktopItem();

	void desktopload(HINSTANCE hInstance, string configLine);
	void desktopreconfigure(string configLine);

	bool isVisible() const { return visible; }

	int getX() const { return actualx; }
	int getY() const { return actualy; }

	const string &getName() const { return name; }

	string getIconCommand() { return iconCommand; }
	
	void repaint(bool invalidateCache = false);
	void updateRegion();

	void reposition(int newx, int newy, int newwidth, int newheight);

	void hide();
	void show();

private:

	HWND hWnd;
	HINSTANCE hInstance;

	HDC backgroundDC;
	HBITMAP backgroundBitmap;

	Texture *background;
	Font *font;

	bool iconOverlay;

	bool visible;

	int actualx;
	int actualy;
	int actualwidth;
	int actualheight;

	bool DIsnapto;
	int DIx;
	int DIy;
	int DIwidth;
	int DIheight;
	int DIsnapdistance;

	int align;
	int vertAlign;

	int leftBorder;
	int topBorder;
	int rightBorder;
	int bottomBorder;

	string name;
	
	string text;

	char tooltip[MAX_LINE_LENGTH];

	bool mousePressed;
	bool mouseInside;

	bool moveable;
	int moveButton;

	bool easyDesktopFocus;

	string leftClickCommand;
	string leftDoubleClickCommand;
	string middleClickCommand;
	string middleDoubleClickCommand;
	string rightClickCommand;
	string rightDoubleClickCommand;
	string iconCommand;
	string dropCommand;

	POINT savedpt;

protected:

	HWND TooltipHints;

	void addHint(HWND hWnd, LPSTR caption);
	void removeHint(HWND hWnd);

	void ontemplateButtonDown(int actualMouseClick);
	void ontemplateButtonUp(const int &x, const int &y, int actualMouseClick, const string &clickCommand);

	int isCloseTo(POINT &pos, const POINT size, int x, int y, const int cx, const int cy, const int span);

	virtual void onLButtonDblClk(int x, int y);
	virtual void onLButtonDown(int x, int y);
	virtual void onLButtonUp(int x, int y);
	virtual void onMButtonDblClk(int x, int y);
	virtual void onMButtonDown(int x, int y);
	virtual void onMButtonUp(int x, int y);
	virtual void onRButtonDblClk(int x, int y);
	virtual void onRButtonDown(int x, int y);
	virtual void onRButtonUp(int x, int y);
	virtual void onMouseMove(int x, int y);
	virtual void onPaint(HDC hDC);
	virtual void onSize(int width, int height);
	virtual void onTimer(int timerID);
	virtual void onWindowPosChanging(WINDOWPOS *windowPos);
	virtual void onWindowPosChanged(WINDOWPOS *windowPos);

	virtual bool onWindowMessage(UINT message, WPARAM wParam, LPARAM lParam, LRESULT &lResult);

public:

	static LRESULT WINAPI windowProcedure(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
};

#endif
