#include "stdafx.h"
#include "Font.h"

Font::Font()
{
	hFont = 0;
}

Font::~Font()
{
	if(hFont)
		DeleteObject(hFont);
}

void Font::configure(const string &prefix)
{
	shadow = GetxRCBool(prefix, "FontShadow", false);
	shadowColor = GetxRCColor(prefix, "FontShadowColor", RGB(128, 128, 128));
	shadowX = GetxRCInt(prefix, "FontShadowX", 1);
	shadowY = GetxRCInt(prefix, "FontShadowY", 1);

	textoutline = GetxRCBool(prefix, "FontOutline", false);
	textoutlineColor = GetxRCColor(prefix, "FontOutlineColor", RGB(128, 128, 128));

	name = GetxRCString(prefix, "Font", "Arial");
	height = GetxRCInt(prefix, "FontHeight", 15, 0);
	color = GetxRCColor(prefix, "FontColor", RGB(0, 0, 0));

	bold = GetxRCBool(prefix, "FontBold", false);
	italic = GetxRCBool(prefix, "FontItalic", false);
	underline = GetxRCBool(prefix, "FontUnderline", false);
}

void Font::apply(HDC hDC, int x, int y, int width, int height, const string &text, unsigned int flags, int vertAlign)
{
	if(hFont == 0)
		createHandle();

	RECT r;

	r.left = x;
	r.top = y;
	r.right = x + width;
	r.bottom = y + height;

	hFont = (HFONT) SelectObject(hDC, hFont);
	int bkMode = SetBkMode(hDC, TRANSPARENT);
	int textColor = SetTextColor(hDC, color);

	DrawTextEx(hDC, (char *) text.c_str(), text.length(), &r, flags | DT_CALCRECT, 0);

	int neededHeight = r.bottom - r.top;
	int vPad;

	switch(vertAlign)
	{
	case ALIGN_TOP:
		vPad = 0;
		break;

	case ALIGN_CENTER:
		vPad = (height - neededHeight) / 2;
		break;

	case ALIGN_BOTTOM:
		vPad = (height - neededHeight);
		break;
	}

	r.left = x;
	r.top = y + vPad;
	r.right = x + width;
	r.bottom = r.top + neededHeight;

	if(shadow && !textoutline)
	{
		OffsetRect(&r, (shadowX + 1) / 2, (shadowY + 1) / 2);
		SetTextColor(hDC, shadowColor);
		DrawTextEx(hDC, (char *) text.c_str(), text.length(), &r, flags | DT_END_ELLIPSIS, 0);
		OffsetRect(&r, -shadowX, -shadowY);
	}
	if (textoutline)
	{
		RECT tmpr;
		CopyRect(&tmpr, &r);
		SetTextColor(hDC, textoutlineColor);

		OffsetRect(&tmpr, 1, 1);
		DrawTextEx(hDC, (char *) text.c_str(), text.length(), &tmpr, flags | DT_END_ELLIPSIS, 0);

		OffsetRect(&tmpr, -1, 0);
		DrawTextEx(hDC, (char *) text.c_str(), text.length(), &tmpr, flags | DT_END_ELLIPSIS, 0);

		OffsetRect(&tmpr, -1, 0);
		DrawTextEx(hDC, (char *) text.c_str(), text.length(), &tmpr, flags | DT_END_ELLIPSIS, 0);

		OffsetRect(&tmpr, 0, -1);
		DrawTextEx(hDC, (char *) text.c_str(), text.length(), &tmpr, flags | DT_END_ELLIPSIS, 0);

		OffsetRect(&tmpr, 0, -1);
		DrawTextEx(hDC, (char *) text.c_str(), text.length(), &tmpr, flags | DT_END_ELLIPSIS, 0);

		OffsetRect(&tmpr, 1, 0);
		DrawTextEx(hDC, (char *) text.c_str(), text.length(), &tmpr, flags | DT_END_ELLIPSIS, 0);

		OffsetRect(&tmpr, 1, 0);
		DrawTextEx(hDC, (char *) text.c_str(), text.length(), &tmpr, flags | DT_END_ELLIPSIS, 0);

		OffsetRect(&tmpr, 0, 1);
		DrawTextEx(hDC, (char *) text.c_str(), text.length(), &tmpr, flags | DT_END_ELLIPSIS, 0);
	}

	SetTextColor(hDC, color);
	DrawTextEx(hDC, (char *) text.c_str(), text.length(), &r, flags | DT_END_ELLIPSIS, 0);

	hFont = (HFONT) SelectObject(hDC, hFont);
	SetBkMode(hDC, bkMode);
	SetTextColor(hDC, textColor);
}

void Font::createHandle()
{
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));

	StringCchCopy(lf.lfFaceName, LF_FACESIZE, name.c_str());
	lf.lfHeight = height;
	lf.lfItalic = italic ? TRUE : FALSE;
	lf.lfUnderline = underline ? TRUE : FALSE;
	lf.lfWeight = bold ? FW_BOLD : FW_NORMAL;
    lf.lfCharSet = DEFAULT_CHARSET;
	lf.lfQuality = NONANTIALIASED_QUALITY;

	if(hFont)
		DeleteObject(hFont);

	hFont = CreateFontIndirect(&lf);
}
